#include "tmesh.h"

#include <fstream>
#include <iostream>

TMesh::TMesh() : verts(0), vertsN(0), tris(0), trisN(0), 
  colors(0),  normals(0) {};

void TMesh::Load(char *fname) {

  ifstream ifs(fname, ios::binary);
  if (ifs.fail()) {
    cerr << "INFO: cannot open file: " << fname << endl;
    return;
  }

  ifs.read((char*)&vertsN, sizeof(int));
  char yn;
  ifs.read(&yn, 1); // always xyz
  if (yn != 'y') {
    cerr << "INTERNAL ERROR: there should always be vertex xyz data" << endl;
    return;
  }
  if (verts)
    delete verts;
  verts = new V3[vertsN];

  ifs.read(&yn, 1); // colors 3 floats
  if (colors)
    delete colors;
  colors = 0;
  if (yn == 'y') {
    colors = new V3[vertsN];
  }

  ifs.read(&yn, 1); // normals 3 floats
  if (normals)
    delete normals;
  normals = 0;
  if (yn == 'y') {
    normals = new V3[vertsN];
  }

  ifs.read(&yn, 1); // texture coordinates 2 floats
  float *tcs = 0; // don't have texture coordinates for now
  if (tcs)
    delete tcs;
  tcs = 0;
  if (yn == 'y') {
    tcs = new float[vertsN*2];
  }

  ifs.read((char*)verts, vertsN*3*sizeof(float)); // load verts

  if (colors) {
    ifs.read((char*)colors, vertsN*3*sizeof(float)); // load colors
  }

  if (normals)
    ifs.read((char*)normals, vertsN*3*sizeof(float)); // load normals

  if (tcs)
    ifs.read((char*)tcs, vertsN*2*sizeof(float)); // load texture coordinates

  ifs.read((char*)&trisN, sizeof(int));
  if (tris)
    delete tris;
  tris = new unsigned int[trisN*3];
  ifs.read((char*)tris, trisN*3*sizeof(unsigned int)); // read tiangles

  ifs.close();

  cerr << "INFO: loaded " << vertsN << " verts, " << trisN << " tris from " << endl << "      " << fname << endl;
  cerr << "      xyz " << ((colors) ? "rgb " : "") << ((normals) ? "nxnynz " : "") << ((tcs) ? "tcstct " : "") << endl;


}

void TMesh::RenderWF(FrameBuffer *fb, PPC *ppc, 
                     unsigned int wfc) {

  V3 *projVerts = new V3[vertsN];
  for (int i = 0; i < vertsN; i++) {
    ppc->Project(verts[i], projVerts[i]);
  }

  for (int i = 0; i < trisN; i++) {
    for (int j = 0; j < 3; j++) {
      int _j = (j+1)%3;
      int vi0  = tris[3*i+j];
      int vi1  = tris[3*i+_j];
      fb->DrawSegment2D(projVerts[vi0], projVerts[vi1], wfc);
    }
  }

  delete projVerts;

}

void TMesh::Translate(V3 trans) {

  for (int i = 0; i < vertsN; i++) {
    verts[i] = verts[i] + trans;
  }

}

V3 TMesh::GetCenter() {

  V3 center(0.0f, 0.0f, 0.0f);
  for (int i = 0; i < vertsN; i++) {
    center = center + verts[i];
  }
  center = center / (float) vertsN;
  return center;

}

AABB TMesh::GetAABB() {

  AABB ret;
  for (int i = 0; i < vertsN; i++) {
    ret.AddPoint(verts[i]);
  }

  return ret;

}

void TMesh::Scale(float diag) {

  AABB aabb = GetAABB();
  float currDiag = (aabb[0]-aabb[1]).Length();
  float scalef = diag / currDiag;
  V3 c = (aabb[0]+aabb[1]) / 2.0f;
  for (int i = 0; i < vertsN; i++) {
    verts[i] = c + (verts[i]-c)*scalef;
  }

}

void TMesh::SetCenter(V3 newCenter) {

  V3 oldCenter = GetCenter();
  Translate(newCenter-oldCenter);

}
